@@ -5,7 +5,7 @@ use warnings;
 package MooX::Press::Keywords;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.018';
+our $VERSION   = '0.019';
 
 use Type::Library -base;
 use Type::Utils ();
@@ -41,10 +41,6 @@ sub confess {
 
 $EXPORT_TAGS{ 'util' } = [qw/ blessed confess /];
 
-use Type::Params qw( compile compile_named compile_named_oo );
-
-$EXPORT_TAGS{ 'params' } = [qw/ compile compile_named compile_named_oo /];
-
 use Try::Tiny;
 
 $EXPORT_TAGS{ 'try' } = [qw/ try catch finally /];
@@ -120,10 +116,6 @@ L<Types::Standard>, L<Types::Common::Numeric>, and L<Types::Common::String>.
 
 =item *
 
-C<compile>, C<compile_named>, and C<compile_named_oo> from L<Type::Params>.
-
-=item *
-
 C<true> and C<false> boolean constants.
 
 =item *
